"use strict";
/**
 * Copyright 2020, Google LLC
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *   http://www.apache.org/licenses/LICENSE-2.0
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.createLocal = exports.filterPackages = exports.filterFiles = exports.Bundler = void 0;
const child_process_1 = require("child_process");
const fs_1 = __importDefault(require("fs"));
const path_1 = __importDefault(require("path"));
require("array.prototype.flatmap/auto");
const replace_in_file_1 = require("replace-in-file");
var Bundler;
(function (Bundler) {
    Bundler["WEBPACK"] = "webpack";
    Bundler["ROLLUP"] = "rollup";
    Bundler["PARCEL"] = "parcel";
    Bundler["NONE"] = "none";
})(Bundler = exports.Bundler || (exports.Bundler = {}));
function filterFiles(selectedTag, ...excludedTags) {
    return (f) => {
        const basename = path_1.default.parse(f).name;
        // exclude.
        for (const excludedTag of excludedTags) {
            if (basename.includes(excludedTag)) {
                return [];
            }
        }
        // rename.
        if (basename.endsWith(`.${selectedTag}`)) {
            return [[f, f.replace(`.${selectedTag}`, "")]];
        }
        // include.
        return [[f, f]];
    };
}
exports.filterFiles = filterFiles;
function filterPackages(packageJson, selectedTag, ...excludedTags) {
    // exclude deps.
    if (packageJson.devDependencies !== undefined) {
        for (const dd of Object.keys(packageJson.devDependencies)) {
            for (const excludedTag of excludedTags) {
                if (dd.includes(excludedTag)) {
                    delete packageJson.devDependencies[dd];
                }
            }
        }
    }
    if (packageJson.scripts !== undefined) {
        for (const s of Object.keys(packageJson.scripts)) {
            // exclude scripts.
            for (const excludedTag of excludedTags) {
                if (s.endsWith(`-${excludedTag}`)) {
                    delete packageJson.scripts[s];
                }
            }
            // rename scripts.
            if (s.endsWith(`-${selectedTag}`)) {
                const script = packageJson.scripts[s].replace(`.${selectedTag}`, "");
                packageJson.scripts[s.replace(`-${selectedTag}`, "")] = script;
                delete packageJson.scripts[s];
            }
        }
    }
    return packageJson;
}
exports.filterPackages = filterPackages;
async function createLocal(selectedTag, srcDir, destDir) {
    const excludedTags = Object.values(Bundler).filter((t) => t !== selectedTag);
    // create app directory.
    await new Promise((resolve, reject) => {
        fs_1.default.mkdir(destDir, (errMkdir) => {
            if (errMkdir !== null) {
                console.error("error creating destination directory:", destDir, errMkdir);
                reject(errMkdir);
            }
            else {
                resolve();
            }
        });
    });
    // get source files.
    const files = await new Promise((resolve, reject) => {
        fs_1.default.readdir(srcDir, (errReaddir, entries) => {
            if (errReaddir !== null) {
                console.error("error reading source directory:", srcDir, errReaddir);
                reject(errReaddir);
            }
            else {
                resolve(Promise.all(entries.map((entry) => new Promise((res, rej) => {
                    fs_1.default.stat(path_1.default.join(srcDir, entry), (errStat, stats) => {
                        if (errStat !== null) {
                            console.error("error getting source file stats:", entry, errStat);
                            rej(errStat);
                        }
                        else {
                            if (stats.isFile()) {
                                res([entry]);
                            }
                            else {
                                res([]);
                            }
                        }
                    });
                }))).then((result) => result.flatMap((x) => x)));
            }
        });
    });
    // filter files to copy.
    const filePairs = files.flatMap(filterFiles(selectedTag, ...excludedTags));
    // copy files
    await Promise.all(filePairs.map(([srcFile, destFile]) => new Promise((resolve, reject) => {
        fs_1.default.readFile(path_1.default.join(srcDir, srcFile), (errReadFile, data) => {
            if (errReadFile !== null) {
                console.error("error reading source file:", srcFile, errReadFile);
                reject(errReadFile);
            }
            else {
                fs_1.default.writeFile(path_1.default.join(destDir, destFile), data, (errWriteFile) => {
                    if (errWriteFile !== null) {
                        console.error("error writing destination file:", destFile, errWriteFile);
                        reject(errWriteFile);
                    }
                    else {
                        resolve();
                    }
                });
            }
        });
    })));
    // search and replace renamed files
    await Promise.all(filePairs
        .filter(([srcFile, destFile]) => (srcFile !== destFile))
        .map(([srcFile, destFile]) => replace_in_file_1.replaceInFile({
        files: path_1.default.join(destDir, "*"),
        from: new RegExp(srcFile, "g"),
        to: destFile,
    })));
    // rewrite package.json deps and rules in-place.
    await new Promise((resolve, reject) => {
        fs_1.default.readFile(path_1.default.join(destDir, "package.json"), (errReadFile, buf) => {
            if (errReadFile !== null) {
                console.error("error reading package file:", errReadFile);
                reject(errReadFile);
            }
            else {
                const packageJson = JSON.parse(buf.toString());
                filterPackages(packageJson, selectedTag, ...excludedTags);
                fs_1.default.writeFile(path_1.default.join(destDir, "package.json"), JSON.stringify(packageJson, null, 4), (errWriteFile) => {
                    if (errWriteFile !== null) {
                        console.error("error writing package file:", errWriteFile);
                        reject(errWriteFile);
                    }
                    else {
                        resolve();
                    }
                });
            }
        });
    });
    // run npm install.
    await new Promise((resolve, reject) => {
        const npmInstall = child_process_1.spawn("npm", ["install", "--prefix", destDir], { stdio: "inherit" });
        let errSpawn;
        npmInstall.on("error", (err) => {
            errSpawn = err;
        });
        npmInstall.on("exit", (code, signal) => {
            if (errSpawn || code !== 0 || signal !== null) {
                console.error("npm install failed:", errSpawn, code, signal);
                reject(errSpawn || new Error("npm install failed"));
            }
            else {
                resolve();
            }
        });
    });
}
exports.createLocal = createLocal;
